/* Free for absolutely any use whatsoever. */

#include <stdio.h>
#include <string.h>
#include "pz.h"
#include "vcard.h"
#include <stdarg.h>


#ifdef IPOD
	#define VCARD_DIR "/Contacts"
#else
	#define VCARD_DIR "/Users/kainjow/Documents/vCards"
#endif


static PzModule *module = NULL;
static PzConfig *config = NULL;
static TWindow *contact_window = NULL;
static TWidget *contacts_menu = NULL;
static vcards_list *vc_list = NULL;
static int current_index = -1;


#define SORT_CONFIG		1
#define DISPLAY_CONFIG	2
static int sort_setting = 0, display_setting = 0;
static const char* settings_options[] = { "Last, First", "First Last", NULL };


extern int asprintf(char **, const char *, ...);

static void show_vcard(vcard *card);


void trim(char *str)
{
	int i;
	for (i=strlen(str)-1; i>=0; i--)
		if (isspace(str[i]))
			str[i] = 0;
		else
			break;
	
	while (str[0] != 0 && isspace(str[0]))
		for (i=0; i<strlen(str); i++)
			str[i] = str[i+1];
}

int empty(char *str)
{
	return (str == NULL || str[0] == 0);
}

void formatted_menu_name(vcard *card, char *formatted, int first_last)
{
	// this function is used for getting a name from the vCard.
	// since not all vCards have first, last and middle names,
	// we have to figure out what it has and create a name based on that
	
	vcard_name *name = card->name;
	
	//printf("first: -->%<-- (%s)\n", name->first, (name->first == NULL ? "null" : "not null"));
	//printf("last: -->%<-- (%s)\n", name->last, (name->last == NULL ? "null" : "not null"));
	//printf("middle: -->%<-- (%s)\n", name->middle, (name->middle == NULL ? "null" : "not null"));
	
	if (name != NULL)
	{
		if (!empty(name->first) && !empty(name->last))
		{
			if (first_last)
			{
				if (empty(name->middle))
					sprintf(formatted, "%s %s", name->first, name->last);
				else
					sprintf(formatted, "%s %s %s", name->first, name->middle, name->last);
			}
			else
			{
				if (empty(name->middle))
					sprintf(formatted, "%s, %s", name->last, name->first);
				else
					sprintf(formatted, "%s, %s %s", name->last, name->first, name->middle);
			}
		}
		else
		{
			sprintf(formatted, "%s", (name->last != NULL ? name->last : (name->first != NULL ? name->first : name->middle)));
		}
	}
	else
	{
		formatted[0] = 0;
	}
}

void append(char **output, char *text)
{
	if (*output == NULL)
		*output = calloc(strlen(text)+1, sizeof(char));
	else
		*output = realloc(*output, strlen(*output) + strlen(text) + 1);
	strcat(*output, text);
}

void appendf(char **output, const char *format, char *v)
{
	char *temp;
	asprintf(&temp, format, v);
	append(output, temp);
	free(temp);
}

// output must not be initialized
void format_vcard(vcard *card, char **output)
{
	// this creates a string showing the vCard's contents

	// name
	char name[100];
	formatted_menu_name(card, name, 1);
	appendf(output, "%s\n", name);

	// organization and title
	if (!empty(card->organization))
	{
		appendf(output, "%s\n", card->organization);
	}
	else if (!empty(card->title))
	{
		appendf(output, "%s\n", card->title);
	}
	
	// phone
	vcard_phone *phone = card->phone;
	if (phone != NULL && (!empty(phone->home) || !empty(phone->mobile) || !empty(phone->work)))
	{
		append(output, "\nTelephone\n");
		if (!empty(phone->home))
		{
			appendf(output, "home: %s\n", phone->home);
		}
		if (!empty(phone->mobile))
		{
			appendf(output, "mobile: %s\n", phone->mobile);
		}
		if (!empty(phone->work))
		{
			appendf(output, "work: %s\n", phone->work);
		}
	}
	
	// email
	if (card->emails != NULL && card->emails->count > 0)
	{
		append(output, "\nE-mail\n");

		int i;
		for (i=0; i<card->emails->count; i++)
		{
			if (!empty(card->emails->addresses[i]))
			{
				appendf(output, "%s\n", card->emails->addresses[i]);
			}
		}
	}
	
	// web
	if (!empty(card->url))
	{
		appendf(output, "\nWeb\n%s\n", card->url);
	}
	
	// office address
	if (!empty(card->work_label))
	{
		appendf(output, "\nOffice address\n%s\n", card->work_label);
	}
	else
	{
		vcard_address *adr = card->work_address;
		if (adr != NULL)
		{
			append(output, "\nOffice address\n");
			
			if (adr->street != NULL)
				appendf(output, "%s\n", adr->street);
	
			if (!empty(adr->city) && !empty(adr->state) && !empty(adr->zip))
			{
				char *s;
				asprintf(&s, "%s, %s %s\n",  adr->city, adr->state, adr->zip);
				append(output, s);
				free(s);
			}

			if (!empty(adr->country))
				appendf(output, "%s\n",  adr->country);
		}
	}

	// home address
	if (!empty(card->home_label))
	{
		appendf(output, "\nHome address\n%s\n", card->work_label);
	}
	else
	{
		vcard_address *adr = card->home_address;
		if (adr != NULL)
		{
			append(output, "\nHome address\n");
			
			if (adr->street != NULL)
				appendf(output, "%s\n", adr->street);
	
			if (!empty(adr->city) && !empty(adr->state) && !empty(adr->zip))
			{
				char *s;
				asprintf(&s, "%s, %s %s\n",  adr->city, adr->state, adr->zip);
				append(output, s);
				free(s);
			}

			if (!empty(adr->country))
				appendf(output, "%s\n",  adr->country);
		}
	}
	
	// birthday
	if (card->birthday != NULL)
	{
		static char *months[] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December", NULL};
		char *s;
		asprintf(&s, "\nBirthday\n%s %d, %d\n",  months[card->birthday->month-1], card->birthday->day, card->birthday->year);
		append(output, s);
		free(s);
	}
	
	// notes
	if (!empty(card->note))
	{
		trim(card->note);
		appendf(output, "\nNotes\n%s\n", card->note);
	}
}


static int textarea_down(TWidget *this, int button)
{
	// here we process button presses when viewing the vCard's info
	
	if (button == TTK_BUTTON_NEXT)
	{
		// show the next vCard in the list
		if (current_index < vc_list->count-1)
			show_vcard(& (vc_list->cards[++current_index]) );			
	}
	else if (button == TTK_BUTTON_PREVIOUS)
	{
		// show the previous vCard in the list
		if (current_index > 0)
			show_vcard(& (vc_list->cards[--current_index]) );			
	}
	else if (button == TTK_BUTTON_MENU)
	{
		// go back to the menu
		ttk_hide_window(contact_window);
	}

    return ttk_menu_down(this, button);
}

static void show_vcard(vcard *card)
{
	// in a summary: creates the contact viewer window if it's not already created (we only use one).
	// then, create a formatted string for the vCard, and trim trailing whitespace.
	// create a new textarea widget, and remove the old one.
	// add the textarea widget to the window, set the window's title, and then display the window!
	
	static TWidget *textarea = NULL;
	char *text = NULL;
	
	if (contact_window == NULL)
		contact_window = ttk_new_window();

	format_vcard(card, &text);
	trim(text);

	if (textarea != NULL)
	{
		ttk_remove_widget(contact_window, textarea);
		ttk_free_widget(textarea);
	}
	textarea = ttk_new_textarea_widget(ttk_screen->w - ttk_screen->wx, ttk_screen->h - ttk_screen->wy, text, ttk_textfont, ttk_text_height(ttk_textfont)+2);
	textarea->down = textarea_down;

	free(text);

	ttk_add_widget(contact_window, textarea);
	ttk_window_set_title(contact_window, "Contact");
	ttk_show_window(contact_window);
}

static int menu_down(TWidget *this, int button)
{
	// process button presses when viewing the menu
	
	if (button == TTK_BUTTON_ACTION)
	{
		// select/action button pressed. so show the info for the selected vCard
		int index = (int)(ttk_menu_get_selected_item(this)->data);
		show_vcard(& (vc_list->cards[index]) );
		// keep track of the vCard's index for later..
		current_index = index;
	}

    return ttk_menu_down(this, button);
}

static int sort_contacts(const void *a, const void *b)
{
	// this is the sort function that gets called from qsort
	// we are sorting based on the Sort setting, so we create
	// formatted names and compare those
	
	vcard *va = (vcard *)a;
	vcard *vb = (vcard *)b;
	
	char an[100], bn[100];
	formatted_menu_name(va, an, sort_setting);
	//printf("an: %s....\n", an);
	formatted_menu_name(vb, bn, sort_setting);
	//printf("bn: %s....\n", bn);
	
	// case-insensitive compare
	return strcasecmp(an, bn);
}

void resort_contacts()
{
	// use stdlib's built-in qsort for sorting our contacts
	qsort(vc_list->cards, vc_list->count, sizeof(vcard), sort_contacts);
}

void refresh_contacts_menu()
{
	// create our menu only once
    if (contacts_menu == NULL)
    {
    	contacts_menu = ttk_new_menu_widget(0, ttk_menufont, ttk_screen->w - ttk_screen->wx, ttk_screen->h - ttk_screen->wy);
    	// override the down func so we can process button presses
        contacts_menu->down = menu_down;
	}
	
	// clear all items in the menu
	ttk_menu_clear(contacts_menu);
	
	// loop through the contacts and generate menus
    int i;
    for (i=0; i<vc_list->count; i++)
    {
	    ttk_menu_item *item = calloc (1, sizeof(struct ttk_menu_item));

		// format the name based on the Display setting
		char itemName[100];
		formatted_menu_name(&(vc_list->cards[i]), itemName, display_setting);

		// set the item's name, and store the card's index in it's data member
		item->name = strdup(itemName);
		item->data = (void *)i;

		// add the item to the menu
		ttk_menu_append(contacts_menu, item);
    }
}

TWindow *setup_contacts_menu()
{
	// create the window that shows the list of contacts
    TWindow *ret = NULL;
    
    // parse all the contacts in our directory
	parse_dir(VCARD_DIR, &vc_list);
	
	if (vc_list == NULL || vc_list->count == 0)
		return  TTK_MENU_DONOTHING;
		
	ret = ttk_new_window();
	
	// sort the list and update the menu
	resort_contacts();
	refresh_contacts_menu();

	// add the menu to our window
	ttk_add_widget(ret, contacts_menu);

    // set the window title
    ttk_window_set_title(ret, "Contacts");
    
    return ret;
}

void setting_choice_changed(struct ttk_menu_item *item, int cdata)
{
	// we want to update the list of contacts each time a setting is changed
	// so we override the choicechange function and update the config ourself
	
	switch (cdata)
	{
		case DISPLAY_CONFIG:
			display_setting = item->choice;
			pz_set_int_setting(config, DISPLAY_CONFIG, display_setting);
			break;

		case SORT_CONFIG:
			sort_setting = item->choice;
			pz_set_int_setting(config, SORT_CONFIG, sort_setting);
		    if (contacts_menu != NULL)
				resort_contacts();
			break;
	}
	
    if (contacts_menu != NULL)
		refresh_contacts_menu();

	pz_save_config(config);	
}

void cleanup_contacts()
{
	if (vc_list != NULL)
		free_vcards_list(vc_list);

	pz_free_config(config);
}

void init_contacts() 
{
	// register our module
    module = pz_register_module("contacts", cleanup_contacts);
    
    // create our menu
	ttk_menu_item *contactsItem = pz_menu_add_action("/Extras/Utilities/Contacts/View", setup_contacts_menu);
	//contactsItem->flags |= TTK_MENU_ICON_SUB;
	
    // setup our settings stuff
    config = pz_load_config(pz_module_get_cfgpath(module, "contacts.conf"));
    sort_setting = pz_get_int_setting(config, SORT_CONFIG);
    display_setting = pz_get_int_setting(config, DISPLAY_CONFIG);

    // create our menus in Settings
	ttk_menu_item *menu = pz_menu_add_setting("/Extras/Utilities/Contacts/Settings/Sort", SORT_CONFIG, config, settings_options);
	menu->choicechanged = setting_choice_changed;
	menu = pz_menu_add_setting("//Extras/Utilities/Contacts/Settings/Display", DISPLAY_CONFIG, config, settings_options);
	menu->choicechanged = setting_choice_changed;
}

PZ_MOD_INIT (init_contacts)