#include <sys/stat.h>
#include "vcard.h"

vcards_list *vclist;
vcard *currentvCard = NULL;

typedef struct TUserData
{
	int indent;
	int inBegin, inEnd, startData;
	char *cardType;
	char *pname;
	int propType;

} TUserData;

void free_vcards_list(vcards_list *list)
{
	int i=0;
	for (i=0; i<list->count; i++)
	{
		vcard *card = &(list->cards[i]);
		
		free(card->name->first);
		free(card->name->last);
		free(card->name->middle);
		free(card->name);
		free(card->organization);
		free(card->title);
		free(card->note);
		if (card->phone != NULL)
		{
			free(card->phone->home);
			free(card->phone->work);
			free(card->phone->mobile);
		}
		free(card->phone);
		free(card->url);
		free(card->home_label);
		free(card->work_label);
		if (card->home_address != NULL)
		{
			free(card->home_address->street);
			free(card->home_address->city);
			free(card->home_address->state);
			free(card->home_address->zip);
			free(card->home_address->country);
		}
		free(card->home_address);
		if (card->work_address != NULL)
		{
			free(card->work_address->street);
			free(card->work_address->city);
			free(card->work_address->state);
			free(card->work_address->zip);
			free(card->work_address->country);
		}
		free(card->work_address);
		
		vcard_emails *emails = card->emails;
		if (emails != NULL)
		{
			int j;
			for (j=0; j<emails->count; j++)
				free(emails->addresses[j]);
			free(card->emails);
		}
		
		free(card->birthday);
	}

   	free(list->cards);
   	free(list);
}

int right(char *str, char *search)
{
	char *find;
	return ((find = strstr(str, search)) && (strcasecmp(find, search) == 0));
}

void remove_backslashes(char *str)
{
	// replaces \: with :
	
	int i;
	for (i=0; i<strlen(str); i++)
	{
		if (str[i] == '\\' && (i < (strlen(str)-1)) && (str[i+1] == ':'))
		{
			// shift all characters past the \ down one
			int j;
			for (j=i; j<strlen(str); j++)
				str[j] = str[j+1];
		}
	}
}

int prop_contains_param(const CARD_Char **params, char *param_name)
{
	const CARD_Char **p = params;
	while (p[0])
	{
		if (strcasecmp(p[0], param_name) == 0 || (p[1] && strcasecmp(p[1], param_name) == 0))
			return 1;
		p += 2;
	}
	return 0;
}

char *nthfield(char *str, char delimiter, int index)
{
	char *ret = (char *)calloc(1, strlen(str)+1);
	int c = 0, i, k=0;
	
	for (i=0; i<strlen(str)+1; i++)
	{
		if (str[i] == delimiter || i==strlen(str)) // found a new part, or at the end of string
		{
			if (c == index) // what we want
			{
				ret[k] = 0;
				return ret;
			}
			
			k = 0;
			c++;
		}			
		else
		{
			ret[k] = str[i];
			k++;
		}
	}
	
	return NULL;
}

void PropHandler(void *userData, const CARD_Char *propName, const CARD_Char **params)
{
	TUserData *ud = (TUserData *) userData;
    char *pname = NULL;
	
	ud->inBegin = 0;
	ud->inEnd = 0;
	ud->startData = 1;
	
	//printf("prop: %s\n", propName);

	if (strcasecmp(propName, "BEGIN") == 0)
	{
		vclist->count++;
		vclist->cards = (vcard *)realloc(vclist->cards, vclist->count*sizeof(vcard));
		currentvCard = &(vclist->cards[vclist->count-1]);
		memset(currentvCard, 0, sizeof(vcard));
		
		/* begin: vcard/vcal/whatever */
		ud->inBegin = 1;
		ud->indent++;
	}
	else if (strcasecmp(propName, "END") == 0)
	{	
		/* end: vcard/vcal/whatever */
        free(ud->cardType);
        ud->cardType = NULL;
		ud->inEnd = 1;
		ud->indent--;
	}
	else
	{
		if (ud->pname != NULL)
			free(ud->pname);
		ud->pname = strdup(propName);
		ud->propType = -1;
		
		
		//if (strcasecmp(propName, "TEL") == 0 ||
		//	strcasecmp(ud->pname, "ADR") == 0 || right(ud->pname, ".ADR"))
		//{
			if (prop_contains_param(params, "HOME"))
				ud->propType = 0;
			else if (prop_contains_param(params, "WORK"))
				ud->propType = 1;
			else if (prop_contains_param(params, "CELL"))
				ud->propType = 2;
		//}
	};
};

void DataHandler(void *userData, const CARD_Char *data, int len)
{
	TUserData *ud = (TUserData *) userData;
    int i;
	assert(ud != NULL);
	
	//printf("data: %s (len=%d)\n", data, len);
	
	if (ud->inBegin)
	{
		if (len > 0)
		{
            ud->cardType = realloc(ud->cardType, len + 1);
            memcpy(ud->cardType, data, len);
            ud->cardType[len] = 0;
		}
	}
	else if (ud->inEnd)
	{
		if (len > 0)
		{
            ud->cardType = realloc(ud->cardType, len + 1);
            memcpy(ud->cardType, data, len);
            ud->cardType[len] = 0;
		}
        else if (ud->cardType)
        {
            free(ud->cardType);
            ud->cardType = NULL;
        };
	}
	else
	{
		if (ud->startData)
        {
		    ud->startData = 0;
        }

		if (len > 0)
		{
			char str[len+1];
			int k=0;
			for (i = 0; i < len; i++)
			{
				CARD_Char c = data[i];
				if (1) //(c >= ' ' && c <= '~') || isspace(c))
				{
					str[k] = c;
					k++;
				}
			}
			str[k] = 0;
			
			//printf("data: %s...\n", str);
			
			
			if (strcasecmp(ud->pname, "N") == 0)
			{
				
				vcard_name *name = calloc(1, sizeof(vcard_name));
				name->last = nthfield(str, ';', 0);
				name->first = nthfield(str, ';', 1);
				name->middle = nthfield(str, ';', 2);
				currentvCard->name = name;
			}
			else if (strcasecmp(ud->pname, "NOTE") == 0)
			{
				currentvCard->note = strdup(str);
			}
			else if (strcasecmp(ud->pname, "TEL") == 0)
			{
				vcard_phone *phone;

				if (currentvCard->phone == NULL)
					phone = calloc(1, sizeof(vcard_phone));
				else
					phone = currentvCard->phone;

				switch (ud->propType)
				{
					case 0: // home
						phone->home = strdup(str);
						break;
					case 1: // work
						phone->work = strdup(str);
						break;
					case 2: // cell
						phone->mobile = strdup(str);
						break;
				}
				currentvCard->phone = phone;
			}
			else if (strcasecmp(ud->pname, "BDAY") == 0)
			{
				if (strlen(str) >= 10)
				{
					vcard_date *birthday = calloc(1, sizeof(vcard_date));
					sscanf(str, "%d-%d-%d", &(birthday->year), &(birthday->month), &(birthday->day));
					currentvCard->birthday = birthday;
				}
			}
			else if (strcasecmp(ud->pname, "ORG") == 0)
			{
				currentvCard->organization = nthfield(str, ';', 0);
			}
			else if (strcasecmp(ud->pname, "TITLE") == 0)
			{
				currentvCard->title = strdup(str);
			}
			else if (strcasecmp(ud->pname, "EMAIL") == 0)
			{
				vcard_emails *emails;
				
				if (currentvCard->emails == NULL)
					emails = calloc(1, sizeof(vcard_emails));
				else
					emails = currentvCard->emails;
					
				char **addresses = emails->addresses;

				emails->count++;
				addresses = realloc(addresses, emails->count);
				
				addresses[emails->count-1] = strdup(str);
				emails->addresses = addresses;
				currentvCard->emails = emails;
			}
			else if (strcasecmp(ud->pname, "URL") == 0 || right(ud->pname, ".URL"))
			{
				currentvCard->url = strdup(str);
				remove_backslashes(currentvCard->url);
			}
			else if (strcasecmp(ud->pname, "ADR") == 0 || right(ud->pname, ".ADR"))
			{
				int type = ud->propType;
				if ((type == 0 && currentvCard->home_address == NULL) || (type == 1 && currentvCard->work_address == NULL))
				{
					//printf("found address: %s...\n", str);
					vcard_address *adrs = calloc(1, sizeof(vcard_address));
								
					adrs->street = nthfield(str, ';', 2);
					adrs->city = nthfield(str, ';', 3);
					adrs->state = nthfield(str, ';', 4);
					adrs->zip = nthfield(str, ';', 5);
					adrs->country = nthfield(str, ';', 6);
					
					switch (type)
					{
						case 0: // home
							currentvCard->home_address = adrs;
							break;
						case 1: // work
							currentvCard->work_address = adrs;
							break;
					}	
					
					//printf("adr: %s...\n", str);				
					/*printf("0: %s\n", nthfield(str, ';', 0));
					printf("full: %s\n", str);
					printf("street: %s\n", adrs->street);
					printf("city: %s\n", adrs->city);
					printf("state: %s\n", adrs->state);
					printf("zip: %s\n", adrs->zip);
					printf("country: %s\n-----------\n", adrs->country);*/
				}
			}
			else if (strcasecmp(ud->pname, "LABEL") == 0)
			{
				switch (ud->propType)
				{
					case 0: // home
						currentvCard->home_label = strdup(str);
						break;
					case 1: // work
						currentvCard->work_label = strdup(str);
						break;
				}
				//printf("label: %s (type: %d)...\n", str, ud->propType);
			}
			else
			{
				//printf("ignored: %s\n", ud->pname);
			}
		}
		//else
			//printf("No data..\n");
	};
};

void parseFile(const char *fname)
{
	TUserData userData;
    CARD_Parser vp = NULL;
    FILE *f = NULL;
	int parseErr = 0;
    int len, rc;
	char buf[512];

    memset(&userData, 0, sizeof(userData));

	/* open test */
    f = fopen(fname, "r+b");
	if (! f)
	{
		printf("Unable to open vCard %s\n", fname);
		return;
	}


	/* allocate parser */
	vp = CARD_ParserCreate(NULL);

	/* initialise */
	CARD_SetUserData(vp, &userData);
	CARD_SetPropHandler(vp, PropHandler);
	CARD_SetDataHandler(vp, DataHandler);

	/* parse */
    len = fread(buf, 1, sizeof(buf), f); 
	while (len > 0)
	{
		rc = CARD_Parse(vp, buf, len, 0);
		if (rc == 0)
		{
			parseErr = 1;
			printf("Error parsing vcard\n");
			break;
		};

		len = fread(buf, 1, sizeof(buf), f); 
	};
    fclose(f);

	/* finalise parsing */
	if (! parseErr)
		CARD_Parse(vp, NULL, 0, 1);
		
	/* free parser */
	CARD_ParserFree(vp);

    /* free up any remaining user data buffers */
    free(userData.cardType);
}

void parse_dir(const char *dirname, vcards_list **vcl)
{
	DIR *dir = NULL;
	struct dirent *mydir;
	struct stat r;
	int status;
	
	status = stat(dirname, &r);
	if (status != 0)
		return;
	
	if ((dir = opendir(dirname)) == NULL)
		return;
	
	*vcl = calloc(1, sizeof(vcards_list));
	vclist = *vcl;
	
	while (mydir = readdir(dir))
	{
		if (right(mydir->d_name, ".vcf"))
		{
			char path[300];
			sprintf(path, "%s/%s", dirname, mydir->d_name);
			//printf("path: %s\n", path);
			parseFile((const char *)path);
		}
	}
	closedir(dir);
}

